/*
# Example 41-1
# The initialization procedure for a loadable package.
*/

/* random.c */
#include <tcl.h>
/*
* Declarations for application-specific command procedures
*/
int RandomCmd(ClientData clientData,
				Tcl_Interp *interp,
				int argc, char *argv[]);
int RandomObjCmd(ClientData clientData,
				Tcl_Interp *interp,
				int objc, Tcl_Obj * CONST objv[]);
/*
* The Initialization procedure that is called when
* the package is loaded.
*/
int Random_Init(Tcl_Interp *interp) {
	/*
	 * Register two variations of random.
	 * The orandom command uses the object interface.
	 */
	Tcl_CreateCommand(interp, "random", RandomCmd,
			(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	Tcl_CreateObjCommand(interp, "orandom", RandomObjCmd,
			(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	/*
	 * Declare that we implement the random package
	 * so scripts that do "package require random"
	 * can load the library automatically. See page 135.
	 */
	Tcl_PkgProvide(interp, "random", "1.1");
	return TCL_OK;
}


/*
# Example 41-2
# The RandomCmd C command procedure.
 */

/*
* RandomCmd --
* This implements the random Tcl command. With no arguments
* the command returns a random integer.
* With an integer valued argument "range",
* it returns a random integer between 0 and range.
*/
int
RandomCmd(ClientData clientData, Tcl_Interp *interp,
		int argc, char *argv[])
{
	int rand, error;
	int limit = 0;
	if (argc > 2) {
		interp->result = "Usage: random ?range?";
		return TCL_ERROR;
	}
	if (argc == 2) {
		error = Tcl_GetInt(interp, argv[1], &limit);
		if (error != TCL_OK) {
			return error;
		}
	}
	rand = random();
	if (limit != 0) {
		rand = rand % limit;
	}
	sprintf(interp->result, "%d", rand);
	return TCL_OK;
}


/*
# Example 41-3
# The RandomCmd C command procedure.
*/

/*
* RandomObjCmd --
* This implements the random Tcl command from
* Example 41-2 using the object interface.
*/
int
RandomObjCmd(ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj * CONST objv[])
{
	Tcl_Obj *resultPtr;
	int rand, error;
	int limit = 0;
	if (objc > 2) {
		Tcl_WrongNumArgs(interp, 1, objv, 
			"Usage: random ?range?");
		return TCL_ERROR;
	}
	if (objc == 2) {
		error = Tcl_GetIntFromObj(interp, objv[1], &limit);
		if (error != TCL_OK) {
			return error;
		}
	}
	rand = random();
	if (limit != 0) {
		rand = rand % limit;
	}
	resultPtr = Tcl_GetObjResult(interp);
	Tcl_SetIntObj(resultPtr, rand);
	return TCL_OK;
}


